/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.commons.net.telnet;

/***
 * Listener interface used for notification that incoming data is
 * available to be read.
 *
 * @see TelnetClient
 * @since 3.0
 ***/
public interface TelnetInputListener
{

    /***
     * Callback method invoked when new incoming data is available on a
     * {@link TelnetClient}'s {@link TelnetClient#getInputStream input stream}.
     *
     * @see TelnetClient#registerInputListener
     ***/
    public void telnetInputAvailable();
}
